package net.darkmist.unicode;

import java.io.UnsupportedEncodingException;

public class Util
{
	private static final String SPACE_PAD[] = new String[]
	{
		"",
		" ",
		"  ",
		"   ",
		"    ",
		"     ",
		"      ",
		"       ",
		"        ",
		"         ",
		"          ",
		"           ",
		"            ",
		"             ",
		"              ",
		"               "
	};
	private static final String SPACE_16	= "                ";
	public static StringBuffer rpad(StringBuffer sb, int origLen, int len)
	{
		int diff = (origLen + len) - sb.length();
		
		for(int chunks=diff>>4;chunks!=0;chunks--)
			sb.append(SPACE_16);
		return sb.append(SPACE_PAD[diff&0x0F]);
	}

	public static StringBuffer rpad(StringBuffer sb, String s, int len)
	{
		int preLen;

		preLen = sb.length();
		sb.append(s);
		return rpad(sb, preLen,len);
	}

	public static String rpad(String s, int len)
	{
		return rpad(new StringBuffer(),s,len).toString();
	}

	private static String byte2hex[] = new String[]
	{
		"00","01","02","03","04","05","06","07",
		"08","09","0A","0B","0C","0D","0E","0F",
		"10","11","12","13","14","15","16","17",
		"18","19","1A","1B","1C","1D","1E","1F",
		"20","21","22","23","24","25","26","27",
		"28","29","2A","2B","2C","2D","2E","2F",
		"30","31","32","33","34","35","36","37",
		"38","39","3A","3B","3C","3D","3E","3F",
		"40","41","42","43","44","45","46","47",
		"48","49","4A","4B","4C","4D","4E","4F",
		"50","51","52","53","54","55","56","57",
		"58","59","5A","5B","5C","5D","5E","5F",
		"60","61","62","63","64","65","66","67",
		"68","69","6A","6B","6C","6D","6E","6F",
		"70","71","72","73","74","75","76","77",
		"78","79","7A","7B","7C","7D","7E","7F",
		"80","81","82","83","84","85","86","87",
		"88","89","8A","8B","8C","8D","8E","8F",
		"90","91","92","93","94","95","96","97",
		"98","99","9A","9B","9C","9D","9E","9F",
		"A0","A1","A2","A3","A4","A5","A6","A7",
		"A8","A9","AA","AB","AC","AD","AE","AF",
		"B0","B1","B2","B3","B4","B5","B6","B7",
		"B8","B9","BA","BB","BC","BD","BE","BF",
		"C0","C1","C2","C3","C4","C5","C6","C7",
		"C8","C9","CA","CB","CC","CD","CE","CF",
		"D0","D1","D2","D3","D4","D5","D6","D7",
		"D8","D9","DA","DB","DC","DD","DE","DF",
		"E0","E1","E2","E3","E4","E5","E6","E7",
		"E8","E9","EA","EB","EC","ED","EE","EF",
		"F0","F1","F2","F3","F4","F5","F6","F7",
		"F8","F9","FA","FB","FC","FD","FE","FF"
	};
	// as we do a ' '+hex frequently this is worth it speed wise
	private static String byte2spaceHex[] = new String[]
	{
		" 00"," 01"," 02"," 03"," 04"," 05"," 06"," 07",
		" 08"," 09"," 0A"," 0B"," 0C"," 0D"," 0E"," 0F",
		" 10"," 11"," 12"," 13"," 14"," 15"," 16"," 17",
		" 18"," 19"," 1A"," 1B"," 1C"," 1D"," 1E"," 1F",
		" 20"," 21"," 22"," 23"," 24"," 25"," 26"," 27",
		" 28"," 29"," 2A"," 2B"," 2C"," 2D"," 2E"," 2F",
		" 30"," 31"," 32"," 33"," 34"," 35"," 36"," 37",
		" 38"," 39"," 3A"," 3B"," 3C"," 3D"," 3E"," 3F",
		" 40"," 41"," 42"," 43"," 44"," 45"," 46"," 47",
		" 48"," 49"," 4A"," 4B"," 4C"," 4D"," 4E"," 4F",
		" 50"," 51"," 52"," 53"," 54"," 55"," 56"," 57",
		" 58"," 59"," 5A"," 5B"," 5C"," 5D"," 5E"," 5F",
		" 60"," 61"," 62"," 63"," 64"," 65"," 66"," 67",
		" 68"," 69"," 6A"," 6B"," 6C"," 6D"," 6E"," 6F",
		" 70"," 71"," 72"," 73"," 74"," 75"," 76"," 77",
		" 78"," 79"," 7A"," 7B"," 7C"," 7D"," 7E"," 7F",
		" 80"," 81"," 82"," 83"," 84"," 85"," 86"," 87",
		" 88"," 89"," 8A"," 8B"," 8C"," 8D"," 8E"," 8F",
		" 90"," 91"," 92"," 93"," 94"," 95"," 96"," 97",
		" 98"," 99"," 9A"," 9B"," 9C"," 9D"," 9E"," 9F",
		" A0"," A1"," A2"," A3"," A4"," A5"," A6"," A7",
		" A8"," A9"," AA"," AB"," AC"," AD"," AE"," AF",
		" B0"," B1"," B2"," B3"," B4"," B5"," B6"," B7",
		" B8"," B9"," BA"," BB"," BC"," BD"," BE"," BF",
		" C0"," C1"," C2"," C3"," C4"," C5"," C6"," C7",
		" C8"," C9"," CA"," CB"," CC"," CD"," CE"," CF",
		" D0"," D1"," D2"," D3"," D4"," D5"," D6"," D7",
		" D8"," D9"," DA"," DB"," DC"," DD"," DE"," DF",
		" E0"," E1"," E2"," E3"," E4"," E5"," E6"," E7",
		" E8"," E9"," EA"," EB"," EC"," ED"," EE"," EF",
		" F0"," F1"," F2"," F3"," F4"," F5"," F6"," F7",
		" F8"," F9"," FA"," FB"," FC"," FD"," FE"," FF"
	};
	public static String byte2hex(int b)
	{
		return byte2hex[b&0xff];
	}

	public static StringBuffer char2hex(StringBuffer sb, char ch)
	{
		return sb.append(byte2hex[(ch>>8)&0xff]).append(byte2hex[ch&0xff]);
	}

	public static StringBuffer dispStr(StringBuffer sb, String s)
	{
		for(int c=0;c<s.length();c++)
		{
			char ch = s.charAt(c);

			switch(ch)
			{
				case '\u0000':
					sb.append("\\0");
					break;
				case '\u0007':
					sb.append("\\a");
					break;
				case '\b':
					sb.append("\\b");
					break;
				case '\f':
					sb.append("\\f");
					break;
				case '\n':
					sb.append("\\n");
					break;
				case '\r':
					sb.append("\\r");
					break;
				case '\t':
					sb.append("\\t");
					break;
				case '\u000B':
					sb.append("\\v");
					break;
				case '\\':
					sb.append("\\");
					break;
				// frequent for us
				case '\uFFFD':
					sb.append("\\uFFFD");
					break;
				default:
					if(ch > '\u007f' || Character.isISOControl(ch) || !Character.isDefined(ch))
					{
						sb.append("\\u");
						char2hex(sb,ch);
					}
					else
						sb.append(ch);
			}	
		}
		return sb;
	}

	public static StringBuffer hexStr(StringBuffer sb, byte[] bytes)
	{
		int len;

		switch((len=bytes.length))
		{
			case 0:
				return sb;
			case 1:
				return sb.append(byte2hex[bytes[0]&0xff]);
		}
		sb.append(byte2hex[bytes[0]&0xff]);
		for(int c=1;c<len;c++)
			//sb.append(' ').append(byte2hex[((int)bytes[c])&0xff]);
			sb.append(byte2spaceHex[bytes[c]&0xff]);
		return sb;
	}

	public static String bytes2utf8(byte[] bytes)
	{
		try
		{
			return new String(bytes, "UTF-8");
		}
		catch(UnsupportedEncodingException e)
		{
			IllegalStateException ise;
			
			ise = new IllegalStateException("UnsupportedEncodingException for \"UTF-8\" which is required by the JVM spec.");
			ise.initCause(e);
			throw ise;
		}
	}

	public static String bytes2ascii(byte[] bytes)
	{
		try
		{
			return new String(bytes, "ASCII");
		}
		catch(UnsupportedEncodingException e)
		{
			IllegalStateException ise;
			
			ise = new IllegalStateException("UnsupportedEncodingException for \"ASCII\" which is required by the JVM spec.");
			ise.initCause(e);
			throw ise;
		}
	}

	public static byte[] str2AsciiBytes(String s)
	{
		try
		{
			return s.getBytes("US-ASCII");
		}
		catch(UnsupportedEncodingException e)
		{
			IllegalStateException ise;
			
			ise = new IllegalStateException("UnsupportedEncodingException for \"US-ASCII\" which is required by the JVM spec.");
			ise.initCause(e);
			throw ise;
		}
	}

	public static byte[] str2Utf16Bytes(String s)
	{
		try
		{
			return s.getBytes("UTF-16BE");
		}
		catch(UnsupportedEncodingException e)
		{
			IllegalStateException ise;
			
			ise = new IllegalStateException("UnsupportedEncodingException for \"UTF-16BE\" which is required by the JVM spec.");
			ise.initCause(e);
			throw ise;
		}
	}

	public static byte[] str2Utf8Bytes(String s)
	{
		try
		{
			return s.getBytes("UTF-8");
		}
		catch(UnsupportedEncodingException e)
		{
			IllegalStateException ise;
			
			ise = new IllegalStateException("UnsupportedEncodingException for \"UTF-8\" which is required by the JVM spec.");
			ise.initCause(e);
			throw ise;
		}
	}

	/**
	 * Duplicate a byte array possibly using a prexistant one.
	 * @param orig The array to copy
	 * @param prev The destination array if any. May be null.
	 * @return a copy of orig in prev is prev is the correct length
	 */
	public static byte[] byteDup(byte[] orig, byte[] prev)
	{
		if(orig==null)
			return null;
		if(prev==null || prev.length != orig.length)
			prev = new byte[orig.length];
		System.arraycopy(orig,0,prev,0,orig.length);
		return prev;
	}
}
